unit RegPersistDsgn;

{ RegPersistDsgn.pas

Copyright (C) 2001  Richard T. Weeks

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Commercial development licenses for all files in the Registry Persistence
Package can be obtained from the author for US$10.  This includes
    RegistryPersist.pas
    FRegistryBrowser.pas
    FRegistryBrowser.dfm
    FPropertyPersist.pas
    FPropertyPersist.dfm
    FSimpleInput.pas
    FSimpleInput.dfm
    RegPersistDsgn.pas

e-mail addresses:
Richard T. Weeks - rtweeks@acm.org
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry, RegistryPersist, DsgnIntf;

procedure Register;

implementation

uses TypInfo, FPropertyPersist, FRegistryBrowser;

type
  TRootKeyRecord = record
    Key: HKEY;
    Name: String[20];
  end;

  TRegRootKeyProperty = class(TEnumProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

  TRegKeyProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TRegPersistEditor = class(TComponentEditor)
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  TPropertyProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

  TPersistComponentProperty = class(TComponentProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

const
  PredefinedHkeyNames: array [0..6] of String = (
    'HKEY_CLASSES_ROOT',
    'HKEY_CURRENT_USER',
    'HKEY_LOCAL_MACHINE',
    'HKEY_USERS',
    'HKEY_PERFORMANCE_DATA',
    'HKEY_CURRENT_CONFIG',
    'HKEY_DYN_DATA'
  );

procedure Register;
begin
  RegisterComponents('Win32', [TRegistryPersist]);
  RegisterPropertyEditor(TypeInfo(HKEY), TRegistryPersist, 'Root',
    TRegRootKeyProperty);
  RegisterPropertyEditor(TypeInfo(String), TRegistryPersist, 'Key',
    TRegKeyProperty);
  RegisterPropertyEditor(TypeInfo(String), TPropertyPersist, 'PropName',
    TPropertyProperty);
  RegisterPropertyEditor(TComponent.ClassInfo, TPropertyPersist, '',
    TPersistComponentProperty);
  RegisterComponentEditor(TRegistryPersist, TRegPersistEditor);
end;

{ TRegKeyProperty }

procedure TRegKeyProperty.Edit;
var
  BrowserForm: TRegistryBrowserForm;
begin
  BrowserForm := TRegistryBrowserForm.Create(nil);
  try
    BrowserForm.Root := TRegistryPersist(GetComponent(0)).Root;
    BrowserForm.SelectedPath := TRegistryPersist(GetComponent(0)).Key;
    if BrowserForm.ShowModal = mrOk then begin
      SetStrValue(BrowserForm.SelectedPath);
    end;
  finally
    BrowserForm.Free;
  end;
end;

function TRegKeyProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paRevertable, paDialog];
end;

{ TRegRootKeyProperty }

function TRegRootKeyProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TRegRootKeyProperty.GetValue: string;
var
  Value: Cardinal;
begin
  Value := GetOrdValue;
  if (Value >= $80000000) and (Value <= $80000006) then
    Result := PredefinedHkeyNames[Value and $7]
  else
    Result := 'USER_ROOT';
end;

procedure TRegRootKeyProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  for I := Low(PredefinedHkeyNames) to High(PredefinedHkeyNames) do
    Proc(PredefinedHkeyNames[I]);
end;

procedure TRegRootKeyProperty.SetValue(const Value: string);
var
  I: Cardinal;
begin
  for I := Low(PredefinedHkeyNames) to High(PredefinedHkeyNames) do
    if PredefinedHkeyNames[I] = Value then
  begin
    SetOrdValue($80000000 or I);
    Break;
  end;
end;

{ TPropertyProperty }

function TPropertyProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paRevertable, paValueList, paSortList];
end;

procedure TPropertyProperty.GetValues(Proc: TGetStrProc);

  procedure Info(const S: String);
  begin
    MessageDlg(S,mtInformation,[mbOk],0);
  end;

var
  Obj: TObject;
  TypeData: PTypeData;
  PropList: array of PPropInfo;
  PropClass: TClass;
  I: Integer;
  ListProperty: Boolean;
begin
  Obj := TPropertyPersist(GetComponent(0)).Component;
  TypeData := GetTypeData(Obj.ClassInfo);
  SetLength(PropList, TypeData^.PropCount);
  SetLength(PropList, GetPropList(Obj.ClassInfo, [tkInteger, tkChar,
    tkEnumeration, tkFloat, tkString, tkSet, tkWChar, tkLString, tkWString,
    tkClass], PPropList(PropList)));
  for I := 0 to Length(PropList) - 1 do begin
    if PropList[I]^.PropType^^.Kind <> tkClass then
      ListProperty := True
    else begin
      TypeData := GetTypeData(PropList[I]^.PropType^);
      PropClass := TypeData^.ClassType;
      ListProperty := PropClass.InheritsFrom(TStrings);
    end;

    if ListProperty then
      Proc(PropList[I]^.Name);
  end;
end;

procedure TPropertyProperty.SetValue(const Value: string);
var
  Obj: TObject;
begin
  Obj := TPropertyPersist(GetComponent(0)).Component;
  if TypInfo.GetPropInfo(Obj.ClassInfo, Value) <> nil then
    SetStrValue(Value);
end;

{ TRegPersistEditor }

procedure TRegPersistEditor.Edit;
var
  Form: TPropertyPersistForm;
begin
  Form := TPropertyPersistForm.Create(nil);
  Form.Execute(GetDesigner, TRegistryPersist(Component));
end;

procedure TRegPersistEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: Edit;
  end;
end;

function TRegPersistEditor.GetVerb(Index: Integer): string;
const
  Verbs: array [0..0] of String = (
    'Define Persistent Properties...'
  );
begin
  Result := Verbs[Index];
end;

function TRegPersistEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

{ TPersistComponentProperty }

procedure TPersistComponentProperty.GetValues(Proc: TGetStrProc);
var
  Root: TComponent;
begin
  inherited GetValues(Proc);

  Root := Designer.GetRoot;
  if Root <> nil then
    Proc(Root.Name);
end;

end.
