unit RegistryPersist;

{ RegistryPersist.pas

Copyright (C) 2001  Richard T. Weeks

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Commercial development licenses for all files in the Registry Persistence
Package can be obtained from the author for US$10.  This includes
    RegistryPersist.pas
    FRegistryBrowser.pas
    FRegistryBrowser.dfm
    FPropertyPersist.pas
    FPropertyPersist.dfm
    FSimpleInput.pas
    FSimpleInput.dfm
    RegPersistDsgn.pas

e-mail addresses:
Richard T. Weeks - rtweeks@acm.org
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry;

type
  TPropertyPersist = class;
  TPropertyPersists = class;
  TRegistryPersist = class;

  TPropertyPersist = class(TCollectionItem)
  private
    FPropertyName: String;
    FComponent: TComponent;
    FValueName: String;
    procedure SetComponent(const Value: TComponent);
    procedure SetPropertyName(const Value: String);
    function GetValueName: String;
  protected
    procedure LoadFrom(Reg: TRegistry);
    procedure SaveTo(Reg: TRegistry);
    procedure UpdateDisplayName;
    function GetDisplayName: String; override;
    function StoreValueName: Boolean;
  published
    property Component: TComponent read FComponent write SetComponent;
    property PropName: String read FPropertyName write SetPropertyName;
    property ValueName: String read GetValueName write FValueName stored StoreValueName;
  end;

  TPropertyPersists = class(TCollection)
  private
    FOwner: TRegistryPersist;
    function GetItems(Index: Integer): TPropertyPersist;
    procedure SetItems(Index: Integer; const Value: TPropertyPersist);
  protected
    function GetOwner: TPersistent; override;
    constructor Create(AOwner: TRegistryPersist);
  public
    function Add: TPropertyPersist;
    function FindItemID(ID: Integer): TPropertyPersist;
    function Insert(Index: Integer): TPropertyPersist;
    property Items[Index: Integer]: TPropertyPersist read GetItems write SetItems; default;
    property Owner: TRegistryPersist read FOwner;
  end;

  TRootKey = (rkCurrentUser, rkCurrentConfig, rkUsers, rkLocalMachine, rkClassesRoot);
  TRegistryPersist = class(TComponent)
  private
    FProperties: TPropertyPersists;
    FRoot: HKEY;
    FKey: String;
    FAutoLoad: Boolean;
    procedure SetProperties(const Value: TPropertyPersists);
    procedure SetKey(const Value: String);
    procedure SetRoot(const Value: HKEY);
    { Private declarations }
  protected
    procedure Loaded; override;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Load;
    procedure Save;
    { Public declarations }
  published
    property AutoLoad: Boolean read FAutoLoad write FAutoLoad;
    property Root: HKEY read FRoot write SetRoot;
    property Key: String read FKey write SetKey;
    property Properties: TPropertyPersists read FProperties write SetProperties;
    { Published declarations }
  end;

implementation

uses TypInfo;

{ TPropertyPersist }

function TPropertyPersist.GetDisplayName: String;
begin
  if (FComponent <> nil) and (FPropertyName <> '') then
    Result := ValueName
  else
    Result := ClassName;
end;

function TPropertyPersist.GetValueName: String;
begin
  if FValueName <> '' then
    Result := FValueName
  else
    Result := Format('%s.%s',[FComponent.Name, FPropertyName]);
end;

procedure TPropertyPersist.LoadFrom(Reg: TRegistry);
var
  ValName: String;
  PropInfo: PPropInfo;

  procedure LoadClass;
  var
    Obj: TObject;
  begin
    Obj := TObject(GetOrdProp(FComponent, PropInfo));
    if Obj is TStrings then begin
      TStrings(Obj).Text := Reg.ReadString(ValName);
    end;
  end;

begin
  ValName := ValueName;
  if Reg.ValueExists(ValName) then begin
    try
      PropInfo := GetPropInfo(FComponent.ClassInfo, FPropertyName);
      case PropInfo^.PropType^^.Kind of
        tkInteger: SetOrdProp(FComponent, PropInfo, Reg.ReadInteger(ValName));
        tkChar: SetOrdProp(FComponent, PropInfo, Reg.ReadInteger(ValName));
        tkEnumeration: SetOrdProp(FComponent, PropInfo, Reg.ReadInteger(ValName));
        tkFloat: SetFloatProp(FComponent, PropInfo, Reg.ReadFloat(ValName));
        tkString, tkLString, tkWString:
          SetStrProp(FComponent, PropInfo, Reg.ReadString(ValName));
        tkWChar: SetOrdProp(FComponent, PropInfo, Reg.ReadInteger(ValName));
        tkClass: LoadClass;
      end;
    except
      {do nothing}
    end;
  end;
end;

procedure TPropertyPersist.SaveTo(Reg: TRegistry);
var
  ValName: String;
  PropInfo: PPropInfo;

  procedure SaveClass;
  var
    Obj: TObject;
  begin
    Obj := TObject(GetOrdProp(FComponent, PropInfo));
    if Obj is TStrings then begin
      Reg.WriteString(ValName, TStrings(Obj).Text);
    end;
  end;

begin
  ValName := ValueName;
  try
    PropInfo := GetPropInfo(FComponent.ClassInfo, FPropertyName);
    case PropInfo^.PropType^^.Kind of
      tkInteger: Reg.WriteInteger(ValName, GetOrdProp(FComponent, PropInfo));
      tkChar: Reg.WriteInteger(ValName, GetOrdProp(FComponent, PropInfo));
      tkEnumeration: Reg.WriteInteger(ValName, GetOrdProp(FComponent,
        PropInfo));
      tkFloat: Reg.WriteFloat(ValName, GetFloatProp(FComponent, PropInfo));
      tkString, tkLString, tkWString:
        Reg.WriteString(ValName, GetStrProp(FComponent, PropInfo));
      tkWChar: Reg.WriteInteger(ValName, GetOrdProp(FComponent, PropInfo));
      tkClass: SaveClass;
    end;
  except
    {do nothing}
  end;
end;

procedure TPropertyPersist.SetComponent(const Value: TComponent);
begin
  FComponent := Value;
  UpdateDisplayName;
end;

procedure TPropertyPersist.SetPropertyName(const Value: String);
begin
  FPropertyName := Value;
  UpdateDisplayName;
end;

function TPropertyPersist.StoreValueName: Boolean;
begin
  Result := FValueName <> '';
end;

procedure TPropertyPersist.UpdateDisplayName;
begin
  if (FComponent <> nil) and (FPropertyName <> '') then
    DisplayName := ValueName
  else
    DisplayName := Copy(ClassName, 2, MaxInt);
end;

{ TPropertyPersists }

function TPropertyPersists.Add: TPropertyPersist;
begin
  Result := TPropertyPersist(inherited Add);
end;

constructor TPropertyPersists.Create(AOwner: TRegistryPersist);
begin
  inherited Create(TPropertyPersist);
  FOwner := AOwner;
end;

function TPropertyPersists.FindItemID(ID: Integer): TPropertyPersist;
begin
  Result := TPropertyPersist(inherited FindItemID(ID));
end;

function TPropertyPersists.GetItems(Index: Integer): TPropertyPersist;
begin
  Result := TPropertyPersist(inherited Items[Index]);
end;

function TPropertyPersists.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TPropertyPersists.Insert(Index: Integer): TPropertyPersist;
begin
  Result := TPropertyPersist(inherited Insert(Index));
end;

procedure TPropertyPersists.SetItems(Index: Integer; const Value: TPropertyPersist);
begin
  inherited Items[Index] := Value;
end;

{ TRegistryPersist }

constructor TRegistryPersist.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FProperties := TPropertyPersists.Create(Self);
  FRoot := HKEY_CURRENT_USER;
end;

destructor TRegistryPersist.Destroy;
begin
  FProperties.Free;
  inherited Destroy;
end;

procedure TRegistryPersist.Load;
var
  Reg: TRegistry;
  I: Integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := FRoot;
    if Reg.OpenKey(FKey, False) then begin
      for I := 0 to FProperties.Count - 1 do
        FProperties[I].LoadFrom(Reg);
    end;
  finally
    Reg.Free;
  end;
end;

procedure TRegistryPersist.Loaded;
begin
  if AutoLoad and not (csDesigning in ComponentState) then
    Load;
end;

procedure TRegistryPersist.Save;
var
  Reg: TRegistry;
  I: Integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := FRoot;
    if Reg.OpenKey(FKey, True) then begin
      for I := 0 to FProperties.Count - 1 do
        FProperties[I].SaveTo(Reg);
    end;
  finally
    Reg.Free;
  end;
end;

procedure TRegistryPersist.SetKey(const Value: String);
begin
  FKey := Value;
end;

procedure TRegistryPersist.SetProperties(const Value: TPropertyPersists);
begin
  FProperties.Assign(Value);
end;

procedure TRegistryPersist.SetRoot(const Value: HKEY);
begin
  if FRoot <> Value then begin
    FRoot := Value;
  end;
end;

end.
